function [hits,kmax,numops1,numops2] = track__ridges2(stat,scalerange,detwidth,cthresh,absiprod_on,gamma);

% track_ridges2: track ridges of UDWT
%
% Usage
%    [hits,kmax,numops1,numops2] = track__ridges2(stat,scalerange,...
%        detwidth,cthresh,absiprod_on,gamma);
%
% Inputs
%  stat		stationary wavelet transform
%  scalerange	range of scales to track over
%  detwidth     search width for tracking 
%  cthresh	minimum length of ridge
%  absiprod_on  0: look for maxima   1: look for maxima of absolute value     
%  gamma	1/f noise exponent	
% 
% Outputs
%  hits		delay locations of ridges
%  kmax	        scale along the ridges
%  numops1	number of ops to order log search
%  numops2	number of ops to do linear search(?)
%
% Description
%
% 	Ridge finding algorithm in which I grow the ridges from
% 	beginning at a coarsest scale and keep growing  the ridge
%       At each I can begin a new ridge if there is a local
%       maxima that is not yet part of a ridge.
%     
%	Similar to track_ridges.m except that I include all
%       the points along a valid ridge. 
%
% See also
%	WTMM.m, BuildSkelMapFast.m in WaveLab .701





		kmax = [];normhits = [];hits = [];
		kmaxnorm = [];
		numops1 = 0;numops2 = 0;

		if absiprod_on
	          stat = abs(stat);
	        end


		N = size(stat,1);
		scalerange = reverse(sort(scalerange));


                for scale = scalerange
		 [maxloc,maxval] = findlocalmax(stat(:,scale+1),absiprod_on);	
		maxloc = maxloc(:)';
		maxval = maxval(:)';
		 eval(sprintf('maxloc%d = maxloc;',scale));
		 eval(sprintf('maxval%d = maxval;',scale));
%		 disp(sprintf('%d maxima at scale %d',length(maxloc),scale));

		end

            
		%start at coarsest scale and work our way down
		% building ridges as we go.
		for scale = scalerange(1:(length(scalerange)-1))
		  eval(sprintf('thisloc = maxloc%d;',scale));
		  eval(sprintf('thisval = maxval%d;',scale));

%		 disp(sprintf('%d maxima at scale %d',length(thisloc),scale));
		
		  for n = 1:length(thisloc)


		    %grow the ridge as far as it will go
		    % if it's long enough then add it to the
		    % list of ridges, including peak scale and
		    % position of peak. 

		    ridgelength = 1;
		    thispos = thisloc(n);
%		    disp(sprintf('thisloc %d n %d',thispos,n));
                    

		    ridgescales = scale;
		    ridgelocs = thispos;


		    for nextscale = (scale-1):-1:min(scalerange)
   		     eval(sprintf('nextloc = maxloc%d;',nextscale));
		     eval(sprintf('nextval = maxval%d;',nextscale));

		    % next two lines borrowed from BuildSkelMap.m in Wavelab .701
		    circdist   = min([ abs(nextloc-thispos) ; abs(nextloc-thispos+N); abs(nextloc-thispos-N) ]);
		    [dist,pos] = min(circdist);

		   if ~isempty(nextloc)	
		     numops1 = numops1 + log2(length(nextloc));
		     numops2 = numops2 + 2*min([dist detwidth]);
                   end

		    if ~isempty(pos)

			if dist <= detwidth
			   thispos = nextloc(pos);
			   ridgelength = ridgelength + 1;
			   ridgescales = [ridgescales; nextscale];
			   ridgelocs = [ridgelocs; thispos];
			   % remove this maxima from the list of maxima
			   eval(sprintf('maxloc%d(pos) = [];',nextscale));
			   eval(sprintf('maxval%d(pos) = [];',nextscale));
	
			else
		          break; % don't keep searching on this ridge

			end
		    end
		  end %for nextscale
		   if ridgelength >= cthresh
			hits = [hits;ridgelocs];
			kmax = [kmax;ridgescales];
   	  	   end
		end % n
	     end % scale

		

% Copyright (c) 1998 Thomas T. Liu




                

       
                







